
use crate::{GITEA_KEY, gitea::{User, Repo, Pull}};

macro_rules! gitea_api {
    ($path: expr) => {
        format!("{}{}?token={}", dotenvy_macro::dotenv!("GITEA_URL", "Gitea url is not set!"), $path, GITEA_KEY)
    };
}

pub async fn get_user() -> User {
    let user = reqwest::get(gitea_api!("/api/v1/user")).await.unwrap();

    if user.status() != 200 {
        panic!("Request failed with code {}", user.status());
    }

    let user = user.bytes().await.unwrap();
    let user = String::from_utf8(user.to_vec()).unwrap();
    serde_json::from_str::<User>(user.as_str()).unwrap()
}

pub async fn get_org_repos(org: String) -> Vec<Repo> {
    let repos = reqwest::get(gitea_api!(format!("/api/v1/orgs/{org}/repos"))).await.unwrap();

    if repos.status() != 200 {
        panic!("Request failed with code {}", repos.status());
    }

    let repos = repos.bytes().await.unwrap();
    let repos = String::from_utf8(repos.to_vec()).unwrap();

    serde_json::from_str::<Vec<Repo>>(repos.as_str()).unwrap()
}

pub async fn get_repo_pulls(org: String, repo: String) -> Vec<Pull> {
    let pulls = reqwest::get(gitea_api!(format!("/api/v1/repos/{org}/{repo}/pulls"))).await.unwrap();

    if pulls.status() != 200 {
        panic!("Request failed with code {}", pulls.status());
    }

    let pulls = pulls.bytes().await.unwrap();
    let pulls = String::from_utf8(pulls.to_vec()).unwrap();

    let parsed = serde_json::from_str(pulls.as_str());
    if parsed.is_err() {
        println!("{pulls}");
    }
    parsed.unwrap()
}

pub async fn get_all_prs(repos: Vec<Repo>) -> Vec<Pull> {
    let mut out: Vec<Pull> = vec![];

    let futures: Vec<_> = 
        repos.iter()
            .map(|x| get_repo_pulls(x.owner.login.clone(), x.name.clone()))
            .collect();
    
    let mut set = tokio::task::JoinSet::new();
    
    for future in futures {
        set.spawn(future);
    }

    while let Some(res) = set.join_next().await {
        let res = res.unwrap();

        for pull in res.iter() {
            out.push(pull.clone());
        }
    }
    
    out
}

/**
   The first one is the never seen before PRs and the 2nd is just the data
 */
pub async fn get_new_prs(old: Vec<Pull>, repos: Vec<Repo>) -> ( Vec<Pull>, Vec<Pull> ) {

    log::debug!("Getting all PRs now");
    
    let data = get_all_prs(repos.clone()).await;
    let mut new_pulls: Vec<Pull> = vec![];
    

    for pull in data.iter() {
        let mut alr_exists = false;
        for old_pull in old.iter() {
            if pull.id == old_pull.id {
                alr_exists = true
            }
        }
        if ! alr_exists {
            new_pulls.push(pull.clone());
        }
    }

    (new_pulls, data)
}

pub fn filter_repos(repos: Vec<Repo>) -> Vec<Repo> {
    repos.iter()
        .filter(|x| (!x.archived) && x.has_pull_requests)
        .cloned()
        .collect()
}